module Scripting
  module Commands

    class Command
      attr_reader :name

      def initialize app, name
        @app = app
        @name = name.to_s.downcase.to_sym
        @description = nil
        @work = lambda {} # noop by default
      end

      def describe &blk
        instance_eval &blk if block_given?
        self
      end

      def description desc=nil
        unless desc.nil?
          @description = desc
        end
        @description
      end

      def work &blk
        @work = blk if block_given?
        @work
      end

      def run! *args
        @app.instance_exec(*args, &work)
      end
    end

    def self.instance_init(instance, *args)
      instance.instance_eval do
        options do
          commands Hash.new
        end

        help do
          $stderr.puts "\nAvailable commands:"
          max_width = options.commands.keys.collect { |key| key.to_s.length }.max
          options.commands.each do |k,v|
            $stderr.puts sprintf("  %*s: %s", max_width, k, v.description)
          end
        end

        work do |*args|
          name = args.shift.downcase.to_sym rescue nil
          run_command! name, *args
        end
      end
    end

    module InstanceMethods
      def command name, &blk
        options.commands[name] = Command.new(self, name).describe(&blk)
      end

      def run_command! name, *args
        command = options.commands[name]
        if command.nil?
          help!
          $stderr.puts "\n\nCommand: #{name} is not known"
          exit
        end

        command.run! *args
      end
    end
  end
end
