module Scripting
  module Pluggable
    module ClassMethods
      def plugins
        @__plugins__ ||= []
      end

      def plugin mod, *args
        current = self.plugins
        unless current.include? mod
          current << mod
          include(mod::InstanceMethods) if mod.const_defined? "InstanceMethods"
          extend(mod::ClassMethods) if mod.const_defined? "ClassMethods"

          if mod.respond_to? :class_init
            mod.class_init(self, *args)
          end
        end
      end
    end

    module InstanceMethods
      def plugins
        self.class.plugins
      end

      def plugin mod, *args
        self.class.plugin mod, *args
        if mod.respond_to? :instance_init
          mod.instance_init(self, *args)
        end
      end
    end

    def self.extended(base)
      base.send(:include, InstanceMethods)
      base.send(:extend, ClassMethods)
    end
  end
end